==> trusted_redirect-8.x-1.0.tar.gz: trusted_redirect/src/EventSubscriber/TrustedRedirectSubscriber.php <==
<?php

namespace Drupal\trusted_redirect\EventSubscriber;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ImmutableConfig;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Trusted redirect subscriber to redirect to trusted hosts.
 */
class TrustedRedirectSubscriber implements EventSubscriberInterface {

  /**
   * Key to store the trusted destination value in the request attributes bag.
   *
   * When trusted destination is found during request handling we stored it to
   * the custom request attributes so it can be used in response. It's necessary
   * since original destination query string argument will be stripped out
   * in RedirectResponseSubscriber executed right after. To prevent spoiling
   * request or introducing some globals, it's stored to the custom request
   * attributes.
   *
   * @var string
   */
  const TRUSTED_DESTINATION_ATTRIBUTE = '_trusted_redirect_trusted_destination';

  /**
   * List of trusted hosts.
   *
   * @var array
   */
  protected $trustedHosts;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   The module configuration.
   */
  public function __construct(ImmutableConfig $config) {
    $this->trustedHosts = $config->get('trusted_hosts');
  }

  /**
   * Evaluates whether destination url is trusted or not.
   *
   * @param string $destination
   *   Destination url.
   *
   * @return bool
   *   Whether destination url is trusted or not.
   */
  protected function isTrustedDestination($destination) {
    $url_info = parse_url($destination);
    return in_array($url_info['host'], $this->trustedHosts);
  }

  /**
   * Process trusted hosts.
   *
   * Process before destination query string parameter is stripped out by
   * RedirectResponseSubscriber due to containing external url.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   The event to process.
   */
  public function onRequestProcessTrustedHosts(GetResponseEvent $event) {
    $request = $event->getRequest();
    $destination = $request->query->get('destination');
    // Treat only when destination url is presented, is external and is trusted.
    if ($destination && UrlHelper::isExternal($destination) && $this->isTrustedDestination($destination)) {
      // Store to custom attributes of the request to be used in response.
      $request->attributes->set(self::TRUSTED_DESTINATION_ATTRIBUTE, $destination);
    }
  }

  /**
   * Redirect to trusted host.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The event to process.
   */
  public function onRespondRedirectToTrustedHost(FilterResponseEvent $event) {
    $response = $event->getResponse();
    if ($response instanceof RedirectResponse) {
      $request = $event->getRequest();
      // Get trusted destination from request custom attributes parameter bag.
      $trusted_destination = $request->attributes->get(self::TRUSTED_DESTINATION_ATTRIBUTE);
      if ($trusted_destination) {
        // Redirect to trusted destination.
        $response->setTargetUrl($trusted_destination);
        $event->stopPropagation();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Priority of 101 to run before RedirectResponseSubscriber which
    // strips external hosts out of destination query string parameter.
    $events[KernelEvents::REQUEST][] = ['onRequestProcessTrustedHosts', 101];
    // Priority of 1 to run before RedirectResponseSubscriber to act before
    // destination parameter is processed and evaluated with exception.
    $events[KernelEvents::RESPONSE][] = ['onRespondRedirectToTrustedHost', 1];
    return $events;
  }

}

==> trusted_redirect-8.x-1.3.tar.gz: trusted_redirect/src/EventSubscriber/TrustedRedirectSubscriber.php <==
<?php

namespace Drupal\trusted_redirect\EventSubscriber;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Trusted redirect subscriber to redirect to trusted hosts.
 */
class TrustedRedirectSubscriber implements EventSubscriberInterface {

  /**
   * Key to store the trusted destination value in the request attributes bag.
   *
   * When trusted destination is found during request handling we stored it to
   * the custom request attributes so it can be used in response. It's necessary
   * since original destination query string argument will be stripped out
   * in RedirectResponseSubscriber executed right after. To prevent spoiling
   * request or introducing some globals, it's stored to the custom request
   * attributes.
   *
   * @var string
   */
  const TRUSTED_DESTINATION_ATTRIBUTE = '_trusted_redirect_trusted_destination';

  /**
   * List of trusted hosts.
   *
   * @var array
   */
  protected $trustedHosts;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   The module configuration.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(ImmutableConfig $config, ModuleHandlerInterface $module_handler) {
    $this->trustedHosts = $config->get('trusted_hosts');
    $this->moduleHandler = $module_handler;
  }

  /**
   * Obtain list of trusted hosts.
   *
   * @return array
   *   List of trusted hosts.
   */
  protected function getTrustedHosts() {
    $trusted_hosts = $this->trustedHosts;
    // Let other modules alter the list of trusted hosts.
    $this->moduleHandler->alter('trusted_redirect_hosts', $trusted_hosts);
    return $trusted_hosts;
  }

  /**
   * Evaluates whether destination url is trusted or not.
   *
   * @param string $destination
   *   Destination url.
   *
   * @return bool
   *   Whether destination url is trusted or not.
   */
  protected function isTrustedDestination($destination) {
    $trusted_hosts = $this->getTrustedHosts();
    $url_info = parse_url($destination);
    return in_array($url_info['host'], $trusted_hosts);
  }

  /**
   * Process trusted hosts.
   *
   * Process before destination query string parameter is stripped out by
   * RedirectResponseSubscriber due to containing external url.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   The event to process.
   */
  public function onRequestProcessTrustedHosts(GetResponseEvent $event) {
    $request = $event->getRequest();
    $destination = $request->query->get('destination');
    // Treat only when destination url is presented, is external and is trusted.
    if ($destination && UrlHelper::isExternal($destination) && $this->isTrustedDestination($destination)) {
      // Store to custom attributes of the request to be used in response.
      $request->attributes->set(self::TRUSTED_DESTINATION_ATTRIBUTE, $destination);
    }
  }

  /**
   * Redirect to trusted host.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The event to process.
   */
  public function onRespondRedirectToTrustedHost(FilterResponseEvent $event) {
    $response = $event->getResponse();
    if ($response instanceof RedirectResponse) {
      $request = $event->getRequest();
      // Get trusted destination from request custom attributes parameter bag.
      $trusted_destination = $request->attributes->get(self::TRUSTED_DESTINATION_ATTRIBUTE);
      if ($trusted_destination) {
        // Redirect to trusted destination.
        $response->setTargetUrl($trusted_destination);
        $event->stopPropagation();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Priority of 101 to run before RedirectResponseSubscriber which
    // strips external hosts out of destination query string parameter.
    $events[KernelEvents::REQUEST][] = ['onRequestProcessTrustedHosts', 101];
    // Priority of 1 to run before RedirectResponseSubscriber to act before
    // destination parameter is processed and evaluated with exception.
    $events[KernelEvents::RESPONSE][] = ['onRespondRedirectToTrustedHost', 1];
    return $events;
  }

}
